import os, csv
import math, numpy
from datetime import datetime

# main 
base_dir = os.getcwd()
# directory to adcirc wind dir 
adcirc_wind_dir = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Puget_Sound_ADCIRC_Results-From_ATKINS\ADCIRC_Model_Runs\Model Runs - ADCIRC - Event List and BCs\ADCIRC_BCs_Wind"

# open files to save wind speed in x and y direction in order to compare with wind file manually
wind_xspd_file = (open("wind_spd_x.csv", 'wb'))
wind_xspd_writer = csv.writer(wind_xspd_file)
wind_yspd_file = (open("wind_spd_y.csv", 'wb'))
wind_yspd_writer = csv.writer(wind_yspd_file)

# open files to save wind speed and wind direction. this is the one needed 
wind_spd_file = (open("wind_spd.csv", 'wb'))
wind_spd_writer = csv.writer(wind_spd_file)
wind_dir_file = (open("wind_dir.csv", 'wb'))
wind_dir_writer = csv.writer(wind_dir_file)


# get date of storm events 
os.chdir(adcirc_wind_dir)
event_list = numpy.genfromtxt('EventList.txt', dtype=None)
event_dates = []
for i, event in enumerate(event_list):
    string_date = event[0] + ' ' + event[1]
    event_dates.append(datetime.strptime(string_date, "%d-%b-%Y %H:%M:%S"))

node = 50 # node number of wind station
post_date = datetime.strptime("31-Dec-1978 23:00:00", "%d-%b-%Y %H:%M:%S")
for storm_num in range(1,151):

    # ignore pre-1979 storms
    tdiff = (event_dates[storm_num-1]-post_date).total_seconds()
    if(tdiff<=0): continue

    print "storm:", storm_num, event_dates[storm_num-1]   
	
    # get wind data 
    datestring = event_dates[storm_num-1].strftime("%Y-%m-%d_%H_%M_%S")
    wind_file = "Wind_UVP__Event_" + str(storm_num).rjust(3, '0') + '_'
    wind_file = wind_file + datestring + "_tm6.txt"
    print wind_file
    wind_data = numpy.genfromtxt(wind_file, dtype=None)

    # append first data as storm_num
    wind_x = []
    wind_x.append(storm_num)
    wind_y = []
    wind_y.append(storm_num)
    wind_spd = []
    wind_spd.append(storm_num)
    wind_dir = []
    wind_dir.append(storm_num)  
    # loop through wind data
    for irec, rec in enumerate(wind_data):
	# ignore wind data after 8 days
        if(len(wind_spd)==193): break

	# get wind data at the initial step, which is not saved in wind_spd and wind_dir
        if(irec==0):
            x_spd = float(rec[2+(node-1)*3])
            y_spd = float(rec[3+(node-1)*3])
            wind_x.append(x_spd)
            wind_y.append(y_spd)
            start_x = x_spd
            start_y = y_spd
	# get next wind data and interpolated into 1 hr interval
        else:
            end_x   = float(rec[2+(node-1)*3])
            end_y   = float(rec[3+(node-1)*3])           
            for i in range(1,4):
                x_spd = start_x + i*(end_x-start_x)/3
                y_spd = start_y + i*(end_y-start_y)/3

                wind_x.append(x_spd)
                wind_y.append(y_spd)
                
                # calculate wind speed and wind direction
                wspd = math.sqrt(x_spd**2 + y_spd**2)
                wdir = 90.0 - math.atan2(y_spd, x_spd)*180.0/math.pi - 180.0
                if wdir<0: wdir = wdir + 360.0

                wind_spd.append(wspd)
                wind_dir.append(wdir)
			
            # reset first data to next data for interpolation purpose
            start_x = end_x
            start_y = end_y
	
    # write time series of wind for this storm
    wind_spd_writer.writerow(wind_spd)
    wind_dir_writer.writerow(wind_dir)
    wind_xspd_writer.writerow(wind_x)
    wind_yspd_writer.writerow(wind_y)
    
os.chdir(base_dir)

# close all files 
wind_xspd_file.close()
wind_yspd_file.close()
wind_spd_file.close()
wind_dir_file.close()
