import os, csv
import numpy

# find col of each transect in TWL_look_up matrix 
def match_tsctid(TWL_matrix_header, tsct_info):

    tsct_map = []

    for tsct in tsct_info:
        tsct_id = int(tsct[1])
        for irec, rec in enumerate(TWL_matrix_header):
            if irec<=3: continue
            tsct_id_in_matrix = rec.split("_")[1]
            if(tsct_id==int(tsct_id_in_matrix)):
                tsct_map.append(irec)
                break
                
    return tsct_map

def single_match_tsctid(TWL_matrix_header, tsct_id):
    for irec, rec in enumerate(TWL_matrix_header):
        if irec<=3: continue
        tsct_id_in_matrix = rec.split("_")[1]
        if(tsct_id==int(tsct_id_in_matrix)):
            tsct_map=irec
            break
                
    return tsct_map
            
# look up TWL in TWL_look_up matrix 
def lookup_twl_wind(itime, bad_swl_file, DWL_matrix, TWL_matrix, swl, wSpd, wDir, icol):

    swl0 = swl
    swl  = min(13.2, swl)    # keep swl above the range to upper limit (this did not occur)  
    wSpd = min(23.0, wSpd)   # keep wind speed above the range to upper limit (this did not occur)  

    for irec, rec in enumerate(TWL_matrix):
        if(irec==0): continue #skip header
        swl_diff = abs(float(rec[1])-swl)
        spd_diff = abs(float(rec[2])-wSpd)
        dir_diff = abs(float(rec[3])-wDir)
        # match input swl, wspd, wdir centred around that in the lookup matrix within half of their resolution 
        if(swl_diff<=0.164042 and spd_diff<=1.0 and dir_diff<=5):
            flag = "TWL"
            twl = max(swl0,float(rec[icol]))
            dwl = max(swl0,float(DWL_matrix[irec][icol]))
	    # output for manual match 
            bad_swl_file.write(str(itime) + ": " + str(swl) + " , " + str(wSpd) + " , " + str(wDir) + " , " + str(twl) + '\n')
            return flag, dwl, twl

    # no match in the look up table
    # if swl is below the range, set twl and flag to null
    if(swl0<8.21):
	dwl = "null"
        twl = "null"
        flag = "null"   
    # if only wind speed is below the range, set twl to swl and flag to "SWL". this maintains high water level with low wind speed 
    else:
	dwl = swl0
        twl = swl0
        flag = "SWL"  
    return flag, dwl, twl


# main 	
meter_to_feet = 3.28084
base_dir = os.getcwd()

# need to create this folder to save TWL time series 
twl_dir = base_dir + os.sep + "TWL_Time_Series" + os.sep + "SELFE_62"
# need to create this folder to save DWL time series 
dwl_dir = base_dir + os.sep + "DWL_Time_Series" + os.sep + "SELFE_62"

# directory to time series of adcirc water level for each transect
adcirc_wl_dir = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Skagit_Island_Whatcom\Return_Period\Statistics\Forcing_Time_Series\ADCIRC_WL"

# directory to runup results 
runup_dir_skagit_gs = r"U:\2028110558\FEMA_Regions\Region_10\WA\Skagit County-Coastal_FY12\Final_Deliverables\Coastal\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations\Georgia_Strait"
runup_dir_skagit_jdf = r"U:\2028110558\FEMA_Regions\Region_10\WA\Skagit County-Coastal_FY12\Final_Deliverables\Coastal\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations\Juan_de_Fuca"
runup_dir_skagit_ps = r"U:\2028110558\FEMA_Regions\Region_10\WA\Skagit County-Coastal_FY12\Final_Deliverables\Coastal\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations\Possession_Sound"
runup_dir_island_jdf = r"U:\2028110558\FEMA_Regions\Region_10\WA\Island County-Coastal_FY12\Final_Deliverables\Coastal\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations\Juan_de_Fuca"
runup_dir_island_ps = r"U:\2028110558\FEMA_Regions\Region_10\WA\Island County-Coastal_FY12\Final_Deliverables\Coastal\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations\Possession_Sound"
runup_dir_whatcom = r"U:\2028110558\FEMA_Regions\Region_10\WA\Whatcom County-Coastal_FY12\Final_Deliverables\Coastal\Puget Sound\Nearshore_Wave_Models\Runup_Setup_TWL_Calculations"

# load transect information 
#tsct_info_file = 'PointID_TSCT_ADCIRC_County.xlsx'
#tsct_book = pyexc.load_book(tsct_info_file)
#tsct_info = tsct_book['Node_Info'].to_array()
#print tsct_info
tsct_info_file = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Skagit_Island_Whatcom\Return_Period\Statistics\PointID_TSCT_ADCIRC_County.csv"
tsct_info = numpy.genfromtxt(tsct_info_file, skip_header=66, delimiter=",")

# load time series of wind 
wind_dir = r"U:\2028110558\FEMA_Regions\Region_10\WA\Puget Sound\Puget_Sound_Wide_Datasets\Skagit_Island_Whatcom\Return_Period\Statistics\Forcing_Time_Series\SELFE_ID_62"
os.chdir(wind_dir)
wind_spd_file = "wind_spd.csv"
wind_spd      = numpy.loadtxt(wind_spd_file, delimiter=",")
wind_dir_file = "wind_dir.csv"
wind_dir      = numpy.loadtxt(wind_dir_file, delimiter=",")

os.chdir(runup_dir_skagit_gs)
# load TWL lookup matrix 
TWL_matrix_file_ska_gs = "TWL.csv"
TWL_matrix_ska_gs      = numpy.genfromtxt(TWL_matrix_file_ska_gs, dtype=None, delimiter=",")
# load DWL lookup matrix 
DWL_matrix_file_ska_gs = "RefWL.csv"
DWL_matrix_ska_gs      = numpy.genfromtxt(DWL_matrix_file_ska_gs, dtype=None, delimiter=",")

os.chdir(runup_dir_skagit_jdf)
# load TWL lookup matrix 
TWL_matrix_file_ska_jdf = "TWL.csv"
TWL_matrix_ska_jdf      = numpy.genfromtxt(TWL_matrix_file_ska_jdf, dtype=None, delimiter=",")

# load DWL lookup matrix 
DWL_matrix_file_ska_jdf = "RefWL.csv"
DWL_matrix_ska_jdf      = numpy.genfromtxt(DWL_matrix_file_ska_jdf, dtype=None, delimiter=",")

os.chdir(runup_dir_skagit_ps)
# load TWL lookup matrix 
TWL_matrix_file_ska_ps = "TWL.csv"
TWL_matrix_ska_ps      = numpy.genfromtxt(TWL_matrix_file_ska_ps, dtype=None, delimiter=",")

# load DWL lookup matrix 
DWL_matrix_file_ska_ps = "RefWL.csv"
DWL_matrix_ska_ps      = numpy.genfromtxt(DWL_matrix_file_ska_ps, dtype=None, delimiter=",")

os.chdir(runup_dir_island_jdf)
# load TWL lookup matrix 
TWL_matrix_file_isl_jdf = "TWL.csv"
TWL_matrix_isl_jdf      = numpy.genfromtxt(TWL_matrix_file_isl_jdf, dtype=None, delimiter=",")

# load DWL lookup matrix 
DWL_matrix_file_isl_jdf = "RefWL.csv"
DWL_matrix_isl_jdf      = numpy.genfromtxt(DWL_matrix_file_isl_jdf, dtype=None, delimiter=",")

os.chdir(runup_dir_island_ps)
# load TWL lookup matrix 
TWL_matrix_file_isl_ps = "TWL.csv"
TWL_matrix_isl_ps      = numpy.genfromtxt(TWL_matrix_file_isl_ps, dtype=None, delimiter=",")

# load DWL lookup matrix 
DWL_matrix_file_isl_ps = "RefWL.csv"
DWL_matrix_isl_ps      = numpy.genfromtxt(DWL_matrix_file_isl_ps, dtype=None, delimiter=",")

os.chdir(runup_dir_whatcom)
# load TWL lookup matrix 
TWL_matrix_file_whatcom = "TWL.csv"
TWL_matrix_whatcom      = numpy.genfromtxt(TWL_matrix_file_whatcom, dtype=None, delimiter=",")

# load DWL lookup matrix 
DWL_matrix_file_whatcom = "RefWL.csv"
DWL_matrix_whatcom      = numpy.genfromtxt(DWL_matrix_file_whatcom, dtype=None, delimiter=",")


os.chdir(base_dir)
# open file to save maximum TWL for each storm and transect
twl_max_filename = "TWL_max_Wind"  + ".csv"
twl_max_file     = open(twl_max_filename, 'wb')
twl_max_writer   = csv.writer(twl_max_file) 

# open file to save maximum DWL for each storm and transect
dwl_max_filename = "DWL_max_Wind"  + ".csv"
dwl_max_file     = open(dwl_max_filename, 'wb')
dwl_max_writer   = csv.writer(dwl_max_file) 

# open file to save maximum SWL for each storm and transect
swl_max_filename = "SWL_max"  + ".csv"
swl_max_file     = open(swl_max_filename, 'wb')
swl_max_writer   = csv.writer(swl_max_file) 

# this was used to save file for checking look up manually  
check_lookup_filename = "TWL_Lookup_List.txt"
check_lookup_file     = open(check_lookup_filename, 'wb')

# this was used to save data above the forcing matrix, no occurrence
above_range_filename = "above_range.txt"
above_range_file     = open(above_range_filename, 'wb')

# loop through each transect
os.chdir(adcirc_wl_dir)
for itsct, tsct in enumerate(tsct_info):
    County, tsct_id, MESH, PointID, ADC_Node = int(tsct[0]), int(tsct[1]), int(tsct[2]), int(tsct[3]), int(tsct[4]) 
    #if(itsct!=0): continue
    fail = 0
    if County==1 and MESH == 1:
        DWL_matrix = DWL_matrix_ska_gs
        TWL_matrix = TWL_matrix_ska_gs
        print str(runup_dir_skagit_gs)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_ska_gs[0], tsct_id)
        except:
            fail = 1
    elif County == 1 and MESH == 2:
        DWL_matrix = DWL_matrix_ska_jdf
        TWL_matrix = TWL_matrix_ska_jdf
        print str(runup_dir_skagit_jdf)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_ska_jdf[0], tsct_id)
        except:
            fail = 1
    elif County == 1 and MESH == 3:
        DWL_matrix = DWL_matrix_ska_ps
        TWL_matrix = TWL_matrix_ska_ps
        print str(runup_dir_skagit_ps)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_ska_ps[0], tsct_id)
        except:
            fail = 1
    elif County == 3 and MESH == 3:
        DWL_matrix = DWL_matrix_isl_ps
        TWL_matrix = TWL_matrix_isl_ps
        print str(runup_dir_island_ps)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_isl_ps[0], tsct_id)
        except:
            fail = 1
    elif County == 3 and MESH == 2:
        DWL_matrix = DWL_matrix_isl_jdf
        TWL_matrix = TWL_matrix_isl_jdf
        print str(runup_dir_island_jdf)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_isl_jdf[0], tsct_id)
        except:
            fail = 1
    else:
        DWL_matrix = DWL_matrix_whatcom
        TWL_matrix = TWL_matrix_whatcom
        print str(runup_dir_whatcom)
        try:
            tsct_col = single_match_tsctid(TWL_matrix_whatcom[0], tsct_id)
        except:
            fail = 1
    if fail == 0: 
        print "transect id:", tsct_id, tsct_col
    
        # open file to save time series of TWL for this transect
        os.chdir(twl_dir)
        twl_filename = "Point" + str(PointID).rjust(4,'0') + "_TWL"  + ".csv"
        twl_file = open(twl_filename, 'wb')
        twl_writer = csv.writer(twl_file)    
        # open file to save flag information(null, swl, twl) for this transect. what does the data in time series of TWL corresponds to
        twl_flag_filename = "Point" + str(PointID).rjust(4,'0') + "_TWL_Flag"  + ".csv"
        twl_flag_file = open(twl_flag_filename, 'wb')
        twl_flag_writer = csv.writer(twl_flag_file)  
   	
        # open file to save time series of DWL for this transect
        os.chdir(dwl_dir)
        dwl_filename = "Point" + str(PointID).rjust(4,'0') + "_DWL"  + ".csv"
        dwl_file = open(dwl_filename, 'wb')
        dwl_writer = csv.writer(dwl_file)  	
        
        # load time series of adcirc water level for this transect
        os.chdir(adcirc_wl_dir)    
        swl_all_file = "Point" + str(PointID).rjust(4,'0') + "_WL"  + ".csv"
        swl_all = numpy.loadtxt(swl_all_file, delimiter=",")
        
        # append tsct_id to first data of max twl
        TWL_max = []
        TWL_max.append("Point_" + str(PointID).rjust(4,'0'))
        DWL_max = []
        DWL_max.append("Point_" + str(PointID).rjust(4,'0'))	
        SWL_max = []
        SWL_max.append("Point_" + str(PointID).rjust(4,'0'))    
   	
        # loop through storm events
        for istorm in range(len(swl_all)):
            storm_id = swl_all[istorm][0]
            #if(storm_id!=39): continue 
            
            # check if storm id matches in swl, wind_spd and wind_dir csv file 
            #print "storm:", swl_all[istorm][0], wind_spd[istorm][0], wind_spd[istorm][0]
            check_lookup_file.write("Point:" + str(PointID) + " Storm:" + str(storm_id)  + '\n')
  		
            # append storm_id to first data of time series of twl
            TWL = []
            TWL.append(storm_id)          
            TWL_flag = []
            TWL_flag.append(storm_id)   
  		
   	    DWL = []
            DWL.append(storm_id)  
    
   	# loop through time series of this storm starting after 2 days 
            for itime in range(49,193):
                swl  = float(swl_all[istorm][itime])*meter_to_feet
                wSpd = float(wind_spd[istorm][itime])
                wDir = float(wind_dir[istorm][itime])
                
                # find the corresponding twl in TWL lookup matrix 
                flag, dwl_found, twl_found = lookup_twl_wind(itime, check_lookup_file, DWL_matrix, TWL_matrix, swl, wSpd, wDir, tsct_col)
                TWL.append(twl_found)
                TWL_flag.append(flag)
                DWL.append(dwl_found)
                #print swl, wSpd, wDir
                
                # write data that above the range, not observed 
                if(swl>13.2):
                    above_range_file.write("Point: " + str(PointID) + "TSCT: " + str(tsct_id) + ", Storm: " + str(storm_id) + ", Time: " + str(itime) + ", SWL: " + str(swl)  + '\n')
                if(wSpd>23):
                    above_range_file.write("Point: " + str(PointID) + "TSCT: " + str(tsct_id) + ", Storm: " + str(storm_id) + ", Time: " + str(itime) + ", wSpd: " + str(wSpd)  + '\n')
    
            # write time series and get the maximum TWL of this storm for this transect 
            twl_writer.writerow(TWL)
            twl_flag_writer.writerow(TWL_flag)
   	    dwl_writer.writerow(DWL)
            TWL_max.append(max([tw for tw in TWL[1:] if tw!="null"]))
   	    DWL_max.append(max([tw for tw in DWL[1:] if tw!="null"]))
            SWL_max.append(max([sw*meter_to_feet for sw in swl_all[istorm][49:] if sw!="null"]))
    
        twl_file.close()
        twl_flag_file.close()
        dwl_file.close()
        
        # write the maximum TWL for this transect 
        swl_max_writer.writerow(SWL_max)
        dwl_max_writer.writerow(DWL_max)
        twl_max_writer.writerow(TWL_max)
        
# close all files 	
check_lookup_file.close()
above_range_file.close()    
swl_max_file.close()   
twl_max_file.close()
dwl_max_file.close()  
        
os.chdir(base_dir)
